@@ -1,10 +1,10 @@
 test_that("illegal initializations are rejected", {
-  expect_error(DiracDistribution$new("42"), class="const_not_numeric")
+  expect_error(DiracDistribution$new("42"), class = "const_not_numeric")
 })
 
 test_that("it has correct distribution name", {
   lue <- DiracDistribution$new(42.0)
-  expect_identical(lue$distribution(), "Const(42)")  
+  expect_identical(lue$distribution(), "Const(42)")
 })
 
 test_that("const values are returned", {
@@ -18,8 +18,8 @@ test_that("const values are returned", {
   expect_error(
     x$quantile(probs = c(0.25, "A", 0.75)), class = "probs_not_numeric"
   )
-  expect_error(x$quantile(
-    probs = c(-0.25, 0.75)), class = "probs_out_of_range"
+  expect_error(
+    x$quantile(probs = c(-0.25, 0.75)), class = "probs_out_of_range"
   )
   expect_identical(x$quantile(probs = 0.22), 42.0)
 })
