@@ -72,4 +72,40 @@ test_that("QALYs are calculated correctly", {
     "QALY", utility = 0.5, interval = as.difftime(365.25 / 2.0, units = "days")
   )
   expect_intol(t1$QALY(), 0.25, 0.01)
+  r <- 3.5 / 100.0
+  t2 <- LeafNode$new(
+    "QALY", utility = 1.0, interval = as.difftime(365.25, units = "days"),
+    ru = r
+  )
+  expect_intol(t2$QALY(), ((1.0 - exp(-r)) / r), 0.001)
+  u <- ConstModVar$new("", "", 1.0)
+  t3 <- LeafNode$new(
+    "QALY", utility = u, interval = as.difftime(365.25, units = "days"),
+    ru = r
+  )
+  expect_intol(t3$QALY(), ((1.0 - exp(-r)) / r), 0.001)
+})
+
+test_that("graphical representation of the node is as expected", {
+  n <- LeafNode$new(label = "leaf")
+  grDevices::pdf(file = NULL)
+  grid::grid.newpage()
+  vp <- grid::viewport()
+  grid::pushViewport(vp)
+  x <- grid::unit(0.5, "npc")
+  y <- grid::unit(0.5, "npc")
+  bb <- n$grob(x = x, y = y, bb = TRUE)
+  expect_s3_class(bb, "unit")
+  expect_length(bb, 4L)
+  ng <- n$grob(x = x, y = y)
+  expect_s3_class(ng, "grob")
+  grid::grid.draw(ng)
+  rg <- grid::rectGrob(
+    x = bb[[1L]], y = bb[[3L]],
+    width = bb[[2L]] - bb[[1L]], height = bb[[4L]] - bb[[3L]],
+    just = c("left", "bottom")
+  )
+  grid::grid.draw(rg)
+  grid::popViewport()
+  grDevices::dev.off()
 })
