@@ -69,18 +69,11 @@ test_that("abortif error message with >1 expression is correct", {
   f <- function(x) {
     abortif(FALSE, x > 2L, FALSE, message = "failed inequality condition")
   }
-  tmpenv <- env(errmsg = "")
-  tryCatch(
+  expect_error(
     f(3L),
-    error = function(e) {
-      assign("errmsg", rlang::cnd_message(e), env = tmpenv)
-    }
+    regexp = "failed inequality condition x > 2L is not FALSE",
+    class = NULL
   )
-  expect_identical(
-    tmpenv$errmsg,
-    "failed inequality condition\n* x > 2L is not FALSE"
-  )
-  expect_true(TRUE)
 })
 
 # tests of abortifnot
@@ -142,18 +135,11 @@ test_that("abortifnot error message with >1 expression is correct", {
   f <- function(x) {
     abortifnot(TRUE, x > 2L, TRUE, message = "failed inequality condition")
   }
-  tmpenv <- env(errmsg = "")
-  tryCatch(
+  expect_error(
     f(1L),
-    error = function(e) {
-      assign("errmsg", rlang::cnd_message(e), env = tmpenv)
-    }
+    regexp = "failed inequality condition x > 2L is not TRUE",
+    class = NULL
   )
-  expect_identical(
-    tmpenv$errmsg,
-    "failed inequality condition\n* x > 2L is not TRUE"
-  )
-  expect_true(TRUE)
 })
 
 # tests of as_numeric
@@ -190,7 +176,7 @@ test_that("as_numeric identifies non ModVar objects", {
   expect_true(is.na(xo))
 })
 
-test_that("as_value detects ModVar and derived objects", {
+test_that("as_numeric detects ModVar and derived objects", {
   # scalar ModVars
   xi <- ModVar$new(description = "", units = "")
   xo <- as_numeric(xi)
@@ -221,10 +207,44 @@ test_that("as_value detects ModVar and derived objects", {
   expect_intol(xo[[4L]], 42.0, 0.01)
 })
 
+# tests of gbp
+test_that("gbp returns in currency format", {
+  x <- list(42L, 2L, 0.002, 4.567, "a")
+  agbp <- gbp(x, p = TRUE)
+  expect_identical(agbp, c("42.00", "2.00", "0.00", "4.57", "NA"))
+})
+
+# tests of class type tests
 test_that("is_class detects missing arguments", {
   expect_error(is_class())
   expect_error(is_class(42L))
-  expect_error(is_class(class_name = "ModVar"))
+  expect_error(is_class(what = "ModVar"))
+})
+
+test_that("is_class supports vectorized arguments", {
+  n1 <- DecisionNode$new("n1")
+  n2 <- ChanceNode$new()
+  n3 <- LeafNode$new("n3")
+  e1 <- Action$new(source = n1, target = n2, label = "e1")
+  e2 <- Reaction$new(source = n2, target = n3)
+  l <- is_class(x = list(n1, n2, n3, e1, e2), what = "Node")
+  expect_identical(l, c(TRUE, TRUE, TRUE, FALSE, FALSE))
+  l <- is_class(x = list(n1, n2, n3, e1, e2), what = "Edge")
+  expect_identical(l, c(FALSE, FALSE, FALSE, TRUE, TRUE))
+  l <- is_class(x = list(n1, n2, n3, e1, e2), what = "DecisionNode")
+  expect_identical(l, c(TRUE, FALSE, FALSE, FALSE, FALSE))
+  l <- is_class(x = list(n1, n2, n3, e1, e2), what = "Action")
+  expect_identical(l, c(FALSE, FALSE, FALSE, TRUE, FALSE))
+  l <- is_class(
+    x = list(n1, n2, n3, e1, e2),
+    what = c("DecisionNode", "Action")
+  )
+  expect_identical(l, c(TRUE, FALSE, FALSE, TRUE, FALSE))
+  l <- is_class(
+    x = list(n1, n2, n3, e1, e2),
+    what = c("DecisionNode", "Node")
+  )
+  expect_identical(l, c(TRUE, TRUE, TRUE, FALSE, FALSE))
 })
 
 test_that("is_ModVar detects non ModVar objects", {
@@ -269,3 +289,84 @@ test_that("is_Arrow detects objects of type Arrow", {
   expect_false(is_Arrow(s))
   expect_true(is_Arrow(a))
 })
+
+test_that("tornado plot checks arguments as expected", {
+  # to is not a data frame
+  to <- 42L
+  expect_error(
+    tornado_plot(to = to, outcome_mean = 0.0),
+    class = "invalid_parameter"
+  )
+  # data frame with missing column
+  to <- data.frame(
+    Description = "p1",
+    Units = "%",
+    LL = 25.0,
+    UL = 29.0,
+    outcome.min = 100.0,
+    stringsAsFactors = FALSE
+  )
+  expect_error(
+    tornado_plot(to = to, outcome_mean = 0.0),
+    class = "invalid_parameter"
+  )
+  # data frame with column of incorrect type
+  to <- data.frame(
+    Description = "p1",
+    Units = "%",
+    LL = 25.0,
+    UL = 29.0,
+    outcome.min = 100.0,
+    outcome.max = FALSE,
+    stringsAsFactors = FALSE
+  )
+  expect_error(
+    tornado_plot(to = to, outcome_mean = 0.0),
+    class = "invalid_parameter"
+  )
+  # data frame with an NA
+  to <- data.frame(
+    Description = "p1",
+    Units = "%",
+    LL = 25.0,
+    UL = NA_real_,
+    outcome.min = 100.0,
+    outcome.max = 120.0,
+    stringsAsFactors = FALSE
+  )
+  expect_error(
+    tornado_plot(to = to, outcome_mean = 0.0),
+    class = "invalid_parameter"
+  )
+  # valid data frame
+  to <- data.frame(
+    Description = "p1",
+    Units = "%",
+    LL = 25.0,
+    UL = 27.0,
+    outcome.min = 100.0,
+    outcome.max = 120.0,
+    stringsAsFactors = FALSE
+  )
+  grDevices::pdf(file = NULL)
+  expect_no_condition(
+    tornado_plot(to = to, outcome_mean = 0.0)
+  )
+  grDevices::dev.off()
+})
+
+
+test_that("vigtable behaves as expected", {
+  # create test data frame
+  df <- data.frame(
+    Strategy = c("Intervention", "Comparator"),
+    Cost = c(2700.0, 2500.0),
+    LYG = c(0.2345, 0.00026),
+    QALY = c(6.5, 6.2),
+    stringsAsFactors = FALSE
+  )
+  # test arguments
+  expect_error(vigtable(x = 3L))
+  # create text for table
+  expect_output(vigtable(df))
+})
